"""
Description:
    A Module to log and store information about a Pokémon tcg collection
Usage:
    To run as a program "python3 main.py"
    To use as a library:
        "from pokemonCardLogger import clss as pcl" for json storage
        "from pokemonCardLogger import clss_alt as pcl" for sqlite storage (pre-alpha experimental)
    To get just the packs and their ids, run "python3 packRef.py"
"""
from pokemonCardLogger import *



import json
import os
import sys
import pickle

pltfrm = sys.platform
home = os.environ["HOME"]
if pltfrm == "linux":
    prog_data = os.path.join(os.path.join(home, ".config"), "POKEMON_TCG_LOG")
elif pltfrm == "win32" or pltfrm == "cygwin":
    prog_data = os.path.join(os.path.join(home, "Documents"), "POKEMON_TCG_LOG")
elif pltfrm == "darwin":
    prog_data = os.path.join(os.path.join(home, "Documents"), "POKEMON_TCG_LOG")
else:
    print("your system is not supported. quitting")
    quit(1)


def migrate_to_pickle(json_file: str, pickle_file: str):
    if not os.path.exists(json_file):
        raise FileNotFoundError
    with open(json_file) as f:
        log_dict = json.load(f)
    with open(pickle_file, "wb") as f:
        pickle.dump(log_dict, f)


def migrate_to_json(pickle_file: str, json_file: str):
    if not os.path.exists(pickle_file):
        raise FileNotFoundError
    with open(json_file, "rb") as f:
        log_dict = pickle.load(f)
    with open(pickle_file, "w") as f:
        json.dump(log_dict, f)


def main():
    print("will you migrate from pickle or json, 1 for pickle 2 for json")
    mode = input(">>> ")
    if mode not in ("1", "2"):
        print("invalid input. try again")
        return main()
    if mode == "1":
        mode = ".pcllog"
        other = ".json"
    else:
        mode = ".json"
        other = ".pcllog"
    print("please enter the name of the user you wish to migrate")
    user = input(">>> ")
    active_file_user = f"{user}{mode}"
    new_file_user = f"{user}{other}"
    new_file_user = os.path.join(prog_data, new_file_user)
    active_file_user = os.path.join(prog_data, active_file_user)
    if mode == ".pcllog":
        migrate_to_json(active_file_user, new_file_user)
    else:
        migrate_to_pickle(active_file_user, new_file_user)


if __name__ == "__main__":
    main()



"""
Description:
    The alternate library version of Pokémon Card Logger using sqlite
    FYI this module is pre alpha and experimental
Usage:
    from pokemonCardLogger import clssalt as pcl
"""
import pickle
import os
import requests
import hashlib
import datetime as dt
# import pdb


class RqHandle:
    """
    Description:
        Handles the pokemonTcgApi data transmission
    """
    card_url = "https://api.pokemontcg.io/v2/cards"
    pack_url = "https://api.pokemontcg.io/v2/sets"

    def __init__(self, api_key: str):
        """
        Description:
            Constructor method
        Parameters:
            :param api_key: the pokemonTcgApi api key
        """
        self.api_key = api_key
        self.headers = {"X-Api-Key": self.api_key}

    def get_card(self, card_id: str):
        """
        Description:
            Requests from pokemonTcgApi the data for a specific card and returns that data as a dictionary
            If the data is bad raises ValueError
        Parameters:
            :param card_id: a string that represents the card according to pokemonTcgApi
            :return: dict of the data from pokemonTcgApi
        """
        data = requests.get(f"{self.card_url}/{card_id}", headers=self.headers)
        if data.ok:
            return data.json()
        else:
            raise ConnectionError

    def get_pack(self, pack_id: str):
        """
        Description:
            Requests from pokemonTcgApi the data for a specific pack and returns that data as a dictionary
            If the data is bad raises ValueError
        Parameters:
            :param pack_id: a string that represents the pack according to pokemonTcgApi
            :return: dict of the data from pokemonTcgApi
        """
        data = requests.get(f"{self.pack_url}/{pack_id}", headers=self.headers)
        if data.ok:
            return data.json()
        else:
            raise ConnectionError

    def get_all_sets(self):
        """
        Description:
            Requests a list of packs from pokemonTcgApi and returns a generator
            The generator yields a tuple with the id of the pack and the packs name
        Parameters:
            :return: generator consisting of a tuple of pack id and pack name
        """
        data = requests.get(self.pack_url, headers=self.headers)
        if data.ok:
            pass
        else:
            raise ConnectionError
        for i in data.json()["data"]:
            yield i["id"], i["name"]

    def __repr__(self):
        return f"RqHandle({self.api_key}"


class DbHandle:
    """
    Description:
        stores and organizes the log data in a sqlite database
    """

    def __init__(self, file: str, psswrd: str, rq: RqHandle):
        """
        Description:
            Constructor method
        Parameters
            :param db_file: the path to the database file
            :param psswrd: the password for the database
            :param rq: an instance of RqHandle
        """
        self.logfile = file
        self.psswrd = psswrd
        self.rq = rq
        self.psswrd_hash = hashlib.sha512(self.psswrd.encode("utf-8")).hexdigest()
        if self.logfile == ":memory:":
            self.logdict = {}
            self.first_run()
        elif os.path.exists(self.logfile):
            with open(self.logfile, "b") as f:
                self.logdict = pickle.load(f)
            self.validate()
        else:
            self.logdict = {}
            self.first_run()
        self.login_setup()

    def first_run(self):
        """
        Description:
            Sets up the database if it was freshly created
        Parameters:
            :return: None
        """
        self.logdict = {"psswrd": self.psswrd_hash, "login_times": [], "log": {}}
        self.save()

    def validate(self):
        """
        Description:
            Validates the database and password combo. if the password doesn't match, raises ValueError
        Parameters:
            :return: None
        """
        if not self.psswrd_hash == self.logdict["psswrd"]:
            raise PermissionError

    def login_setup(self):
        """
        Description:
            Logs the current login to the database
        Parameters:
            :return: None
        """
        date = dt.datetime.now().isoformat()
        self.logdict["login_times"].append(date)
        self.save()

    def add_card(self, card_id: str, qnty: int):
        """
        Description:
            Adds quantity to the card as well as adds a new card to the database
        Parameters:
            :param card_id: the id of the card according to pokemonTcgApi
            :param qnty: the quantity of cards to add. if there is already quantity, it adds to that
            :return: None
        """
        if not self.test_card(card_id):
            return False
        current_qnty = self.get_card_qnty(card_id)
        if not current_qnty:
            self.logdict["log"].update({card_id: qnty})
        else:
            qnty = qnty + current_qnty
            self.logdict["log"].update({card_id: qnty})
        self.save()
        return True

    def remove_card(self, card_id: str, qnty: int):
        """
        Description:
            Removes quantity from a card in the log
        Parameters:
            :param card_id: the id of the card according to pokemonTcgApi
            :param qnty: the quantity of cards to remove. if there is already quantity, it subtracts from that
            :return: a bool based on if the operation was successful or not
        """
        if not self.test_card(card_id):
            return False
        current_qnty = self.get_card_qnty(card_id)
        if not current_qnty:
            return False
        qnty = current_qnty - qnty
        if qnty < 0:
            qnty = 0
        self.logdict["log"].update({card_id: qnty})
        self.save()
        return True

    def delete_card(self, card_id: str):
        """
        Description:
            Deletes a card from the log
        Parameters:
            :param card_id: the id of the card according to pokemonTcgApi
            :return: None
        """
        if not self.test_card(card_id):
            return False
        _ = self.logdict["log"].pop(card_id)
        self.save()
        return True

    def get_card_qnty(self, card_id: str):
        """
        Description:
            Gets and returns the quantity of a given card in the log
        Parameters
            :param card_id: the id of the card according to pokemonTcgApi
            :return: The quantity of the card
        """
        if not self.test_card(card_id):
            return 0
        try:
            return self.logdict["log"][card_id]
        except KeyError:
            return 0

    def get_log(self):
        """
        Description:
            A generator consisting of the log
        Parameters:
            :return: a generator of the rows in the log
        """
        for card_id, qnty in self.logdict["log"].items():
            yield qnty, card_id

    def test_card(self, card_id: str):
        """
        Description:
            Test if a card id is valid
        Parameters:
            :param card_id: the id of the card according to pokemonTcgApi
            :return: bool if the card is valid or not
        """
        try:
            _ = self.rq.get_card(card_id)
            return True
        except ConnectionError:
            return False

    def close(self):
        self.save()
        quit()

    def save(self):
        """
        Description:
            cleanly closes the log by saving the log to a file
        Parameters:
            :return: None
        """
        pop_items = []
        for card, qnty in self.logdict["log"].items():
            if qnty == 0:
                pop_items.append(card)
        for i in pop_items:
            _ = self.logdict["log"].pop(i)
        if self.logfile == ":memory:":
            return None
        with open(self.logfile, "wb") as f:
            pickle.dump(self.logdict, f)

    def __repr__(self):
        return f"DbHandle('{self.logfile}', 'redacted', {self.rq.__repr__()})"

    def __len__(self):
        return len(list(self.get_log()))


if __name__ == "__main__":
    import config

    print("this is for testing purposes")
    _file = ":memory:"
    _psswrd = "default"
    _rq = RqHandle(config.API_KEY)
    db = DbHandle(_file, _psswrd, _rq)
    print(db.__repr__())



"""
Description:
    the main program for Pokémon card logger
Usage:
    To run as a program "python3 main.py"
    Fill out the prompts to use.
"""
import sys
# noinspection PyUnresolvedReferences
from getpass import getpass
import os
import clss
import clss_alt

try:
    from config import *
except ImportError:
    API_KEY = ""

    if __name__ != "__main__":

        def init(api_key: str):
            global API_KEY
            API_KEY = api_key
    else:
        print("Please enter you pokemontcgapi key: ")
        API_KEY = input(">>> ")

pltfrm = sys.platform
home = os.environ["HOME"]
if pltfrm == "linux":
    prog_data = os.path.join(os.path.join(home, ".config"), "POKEMON_TCG_LOG")
elif pltfrm == "win32" or pltfrm == "cygwin":
    prog_data = os.path.join(os.path.join(home, "Documents"), "POKEMON_TCG_LOG")
elif pltfrm == "darwin":
    prog_data = os.path.join(os.path.join(home, "Documents"), "POKEMON_TCG_LOG")
else:
    print("your system is not supported. quitting")
    quit(1)
try:
    os.makedirs(prog_data)
except FileExistsError:
    pass


def get_card_id(rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Asks the user for a card id and returns the data received from the pokemonTcgApi
    Parameters:
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: the card id from pokemonTcgApi or False if it errors out
    """
    print(
        "please type the pack id of the card. if you dont know what that is run the 5th option from the main menu:"
    )
    pack = input(">>> ")
    try:
        _ = rq.get_pack(pack)
    except ConnectionError:
        print("invalid pack id. try main menu item 5")
        return False
    print("please enter the cards collectors number")
    num = input(">>> ")
    card_id = f"{pack}-{num}"
    try:
        _ = rq.get_card(card_id)
    except ConnectionError:
        print("Error. try again")
        return False
    return card_id


def list_packs(rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
        Description:
            Prints out to console, the list of packs and their pack ids
        Parameters:
            :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
            :return: None
    """
    for pack_id, name in rq.get_all_sets():
        print(f"the pack {name}'s id is: {pack_id}")


def get_mode():
    """
    Description:
        Asks the user what they wish to do
    Parameters:
        :return: a string stating the option chose by the user
    """
    info = """
please select one of the following:
0: quit
1: get card count
2: add card
3: remove a card from count
4: delete card from database
5: list packs
6: list log
7: log size
"""
    print(info)
    mode = input(">>> ")
    if mode == "0":
        return "end"
    elif mode == "1":
        return "card"
    elif mode == "2":
        return "add"
    elif mode == "3":
        return "remove"
    elif mode == "4":
        return "delete"
    elif mode == "5":
        return "packs"
    elif mode == "6":
        return "log"
    elif mode == "7":
        return "len"
    else:
        print("invalid entry try again")
        return get_mode()


def get_card_log(db: (clss.DbHandle, clss_alt.DbHandle), rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Prints to console the list of the log data
    Parameters:
        :param db: an instance of pokemonCardLogger.clss.DbHandle
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: None
    """
    for qnty, card_id in db.get_log():
        data = rq.get_card(card_id)["data"]
        name = data["name"]
        pack = data["set"]["name"]
        print(f"card name: {name}; the pack of the card is: {pack}; count: {qnty}")


def get_card(db: (clss.DbHandle, clss_alt.DbHandle), rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Prints out to the console the data in the log of a specific card
    Parameters:
        :param db: an instance of pokemonCardLogger.clss.DbHandle
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: None
    """
    card_id = get_card_id(rq)
    if not card_id:
        print("canceled")
        return
    card_name = rq.get_card(card_id)["data"]["name"]
    print(f"is {card_name} the name of the card?('y' or 'n')")
    truth = input(">>> ")
    if truth.lower() == "n":
        print("then try again.")
        return
    qnty = db.get_card_qnty(card_id)
    data = rq.get_card(card_id)["data"]
    name = data["name"]
    pack = data["set"]["name"]
    print(f"the card {name} in pack {pack} quantity is: {qnty}")


def add_card(db: (clss.DbHandle, clss_alt.DbHandle), rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Adds more to the value of a specific card count to the log
    Parameters:
        :param db: an instance of pokemonCardLogger.clss.DbHandle
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: None
    """
    card_id = get_card_id(rq)
    if not card_id:
        print("canceled")
        return None
    card_name = rq.get_card(card_id)["data"]["name"]
    print(f"is {card_name} the name of the card?('y' or 'n')")
    truth = input(">>> ")
    if truth.lower() == "n":
        print("then try again.")
        return
    print("how many would you like to add")
    new_count = input(">>> ")
    try:
        new_count = int(new_count)
    except ValueError:
        print("invalid entry. please try again and enter a number")
        return add_card(db, rq)
    success = db.add_card(card_id=card_id, qnty=new_count)
    print(f"the process was successful: {success}")


def remove_card(db: (clss.DbHandle, clss_alt.DbHandle), rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Remove from the value of a specific card count to the log
    Parameters:
        :param db: an instance of pokemonCardLogger.clss.DbHandle
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: None
    """
    card_id = get_card_id(rq)
    if not card_id:
        print("canceled")
        return None
    card_name = rq.get_card(card_id)["data"]["name"]
    print(f"is {card_name} the name of the card?('y' or 'n')")
    truth = input(">>> ")
    if truth.lower() == "n":
        print("then try again.")
        return
    print("how many would you like to remove")
    new_count = input(">>> ")
    try:
        new_count = int(new_count)
    except ValueError:
        print("invalid entry. please try again and enter a number")
        return remove_card(db, rq)
    success = db.remove_card(card_id, new_count)
    print(f"the process was successful: {success}")


def delete_card(db: (clss.DbHandle, clss_alt.DbHandle), rq: (clss.RqHandle, clss_alt.RqHandle)):
    """
    Description:
        Deletes all data from a card in the log
    Parameters:
        :param db: an instance of pokemonCardLogger.clss.DbHandle
        :param rq: an instance of pokemonCardLogger.clss.RqHandle or pokemonCardLogger.clss_alt.RqHandle
        :return: None
    """
    card_id = get_card_id(rq)
    if not card_id:
        print("canceled")
        return
    card_name = rq.get_card(card_id)["data"]["name"]
    print(f"is {card_name} the name of the card?('y' or 'n')")
    truth = input(">>> ")
    if truth.lower() == "n":
        print("then try again.")
        return
    print(" are you sure you want to do this? it cannot be undone.")
    truth = input("(yes/no)>>> ")
    if truth == "yes":
        success = db.delete_card(card_id)
        print(f"the process was successful: {success}")
    else:
        print("canceled")
        return


def get_user():
    """
    Description:
        Gets user data from user, and gives instances of the RqHandle and DbHandle objects
    Parameters
        :return: a tuple of two items consisting of instances of RqHandle and DbHandle
    """
    print("please enter 1 for json or 2 for pickle (pickle is still pre alpha and experimental)")
    mode = input(">>> ")
    if mode == "1":
        rq = clss.RqHandle(API_KEY)
    elif mode == "2":
        rq = clss_alt.RqHandle(API_KEY)
    else:
        print("invalid input. please enter 1 or 2")
        return get_user()
    print("please enter the name of the user, 'default' for the default insecure login")
    user = input(">>> ")
    if mode == "1":
        ext = ".json"
    elif mode == "2":
        ext = ".pcllog"
    user = f"{user}{ext}"
    user_file = os.path.join(prog_data, user)
    if user == "default.json" or user == "default.pcllog":
        psswrd = "default"
    else:
        print("Please enter password for said user.")
        psswrd = getpass(">>> ")
    try:
        if mode == "1":
            db = clss.DbHandle(user_file, psswrd, rq)
        elif mode == "2":
            db = clss_alt.DbHandle(user_file, psswrd, rq)
    except PermissionError:
        print("Invalid password, try again.")
        return get_user()
    return db, rq


def len_of_log(db: (clss.DbHandle, clss_alt.DbHandle)):
    print(f"the size of your logged collection is {len(db)}")


def main():
    """
    Description
        Main Loop
    Parameters:
        :return: None
    """
    db, rq = get_user()
    while True:
        mode = get_mode()
        if mode == "add":
            add_card(db, rq)
        elif mode == "remove":
            remove_card(db, rq)
        elif mode == "card":
            get_card(db, rq)
        elif mode == "delete":
            delete_card(db, rq)
        elif mode == "packs":
            list_packs(rq)
        elif mode == "log":
            get_card_log(db, rq)
        elif mode == "len":
            len_of_log(db)
        elif mode == "end":
            break
    db.close()


if __name__ == "__main__":
    main()



"""
Description:
    A script to display the Pokémon packs and their pack ids according to the logger.
Usage:
    Run with "python3 packRef.py" in a shell to get a list of packs and their pack ids
"""
import clss
try:
    import config
    key = config.API_KEY
except ImportError:
    print("please enter your api key for pokemonTcgApi")
    key = input(">>> ")


if __name__ == "__main__":
    rq = clss.RqHandle(key)
    for pack_id, pack_name in rq.get_all_sets():
        print(f"the pack {pack_name}'s id is {pack_id}")
else:
    print("Not importable, quitting")
    raise NotImplementedError



from setuptools import setup, find_packages
import os

VERSION = '0.1.2'
DESCRIPTION = 'Log Pokemon cards'
LONG_DESCRIPTION = 'a module to log and manage a pokemon card collection'

# Setting up
setup(
    name="pokemonCardLogger",
    version=VERSION,
    author="rosejustin601 (Justin Rose)",
    author_email="rosejustin601@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[],
    keywords=['python', 'pokemon', 'card', 'tcg'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Environment :: Console",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English"
    ]
)
